#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#
import logging
import os
from cicd.informatica import infaSettings
from supporting import errorcodes, executeCommand
from cicd.informatica import infaConstants as constants

logger = logging.getLogger(__name__)


def execute(command, source_or_target=constants.CREATEARTIFACT, pre_command=None):
    """Execute an Informatica command line
    Sets INFA_DEFAULT_DOMAIN_PASSWORD, INFA_DEFAULTS_DOMAIN_USER and INFA_DEFAULT_SECURITY_DOMAIN based on provided Informatica settings.
    """
    if source_or_target == constants.CREATEARTIFACT:
        infa_env = {**os.environ, 'INFA_DEFAULT_DOMAIN_PASSWORD': infaSettings.sourcePassword,
                    'INFA_DEFAULT_DOMAIN_USER': infaSettings.sourceUsername,
                    'INFA_DEFAULT_SECURITY_DOMAIN': infaSettings.sourceSecurityDomain}

    if source_or_target == constants.DEPLOYARTIFACT:
        infa_env = {**os.environ, 'INFA_DEFAULT_DOMAIN_PASSWORD': infaSettings.targetPassword,
                    'INFA_DEFAULT_DOMAIN_USER': infaSettings.targetUsername,
                    'INFA_DEFAULT_SECURITY_DOMAIN': infaSettings.targetSecurityDomain}

    result = executeCommand.execute(command=command, env=infa_env, pre_command=pre_command)

    if result.code == errorcodes.COMMAND_FAILED:
        old_result = result.message
        result = errorcodes.INFACMD_FAILED
        result.message = old_result

    return result



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings
from cicd.informatica import infaConstants
from supporting import generalSettings
from cicd.informatica import manageConnection
import sys, argparse

now = datetime.datetime.now()
result = errorcodes.OK


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """
    parser = argparse.ArgumentParser()
    parser.add_argument("-o", "--outputfile", required=False, action="store", dest="output_file",
                        help="File to export the connection definitions to. Tip: Include the path. Default is >"
                             + infaConstants.DEFAULT_EXPORT_CONNECTIONSFILE + ".")
    parser.add_argument("-e", "--exportcontrolfile", required=False, action="store", dest="export_control_file",
                        help="The export control file allows filtering and more. Check the Informatica documentation on the structure and possibilities. "
                             + "Tip: Include a path.")
    args = parser.parse_args()

    if args.output_file is None:
        args.output_file = infaConstants.DEFAULT_EXPORT_CONNECTIONSFILE

    if args.export_control_file is None:
        args.export_control_file = ""

    return args


def main(argv):
    """Exports the connection definitions from the Informatica Domain.
    Usage: exportConnections.py [-h] [-o OUTPUT_FILE] [-e EXPORT_CONTROL_FILE]
    If no output file is provided, the default is set as per infaConstants.DEFAULT_EXPORT_CONNECTIONSFILE
    For information about the export control file, check the Informatica documentation.
    """
    thisproc = "MAIN"
    mainProc = 'exportConnections'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    generalSettings.getenvvars()

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

    output_file = args.output_file
    export_control_file = args.export_control_file

    infaSettings.getinfaenvvars()
    infaSettings.outinfaenvvars()

    if export_control_file != "":
        connection = manageConnection.ManageConnection(Tool="ExportConnections",
                                                       Domain=infaSettings.sourceDomain,
                                                       ExportControlfile=export_control_file,
                                                       ExportFile=output_file,
                                                       RetainPassword='true',
                                                       Force='true',
                                                       OnError=errorcodes.INFACMD_EXPORT_CONN_FAILED
                                                       )
    else:
        connection = manageConnection.ManageConnection(Tool="ExportConnections",
                                                       Domain=infaSettings.sourceDomain,
                                                       ExportFile=output_file,
                                                       RetainPassword='true',
                                                       Force='true',
                                                       OnError=errorcodes.INFACMD_EXPORT_CONN_FAILED
                                                       )

    result = connection.manage()

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv[1:])



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings
from supporting import generalSettings
from cicd.informatica import manageSecurity
import sys, argparse
from supporting.randomize import randomStringDigits

now = datetime.datetime.now()
result = errorcodes.OK


def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """
    parser = argparse.ArgumentParser()
    parser.add_argument("-u", "--username", required=True, action="store", dest="username",
                        help="User to be created.")
    parser.add_argument("-p", "--password", required=False, action="store", dest="password",
                        help="User password. If not provided a password will be generated.")
    parser.add_argument("-f", "--fullname", required=True, action="store", dest="fullname",
                        help="User full name.")
    parser.add_argument("-d", "--description", required=False, action="store", dest="description",
                        help="A description for the user. If not provided, the description will be generated with the creation timestamp.")
    parser.add_argument("-e", "--email", required=False, action="store", dest="email",
                        help="The email address of the user.")
    parser.add_argument("-n", "--phone", required=False, action="store", dest="phonenumber",
                        help="The user's phone number.")
    args = parser.parse_args()

    if args.password is None:
        args.password = randomStringDigits(12)
        generated_password = True
    else:
        generated_password = False

    if args.description is None:
        args.description = "Created by createUser.py on " + now.strftime('%Y-%m-%d %H:%M:%S.%f')
    if args.email is None:
        args.email = ""
    if args.phonenumber is None:
        args.phonenumber = ""

    return args, generated_password


def main(argv):
    """Create a user.
    If a password is not provided, one will be generated
    usage: createUser.py [-h] -u USERNAME [-p PASSWORD] -f FULLNAME
                     [-d DESCRIPTION] [-e EMAIL] [-n PHONENUMBER]
    """
    thisproc = "MAIN"
    mainProc = 'createUser'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args, generated_password = parse_the_arguments(argv)

    generalSettings.getenvvars()

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

    user_name = args.username
    user_password = args.password
    user_fullname = args.fullname
    user_description = args.description
    user_email = args.email
    user_phone = args.phonenumber

    if generated_password:
        print(args.password)

    infaSettings.getinfaenvvars()
    infaSettings.outinfaenvvars()

    user = manageSecurity.ManageSecurity(Tool="CreateUser",
                                         Domain=infaSettings.sourceDomain,
                                         NewUserName=user_name,
                                         NewUserPassword=user_password,
                                         NewUserFullName=user_fullname,
                                         NewUserDescription=user_description,
                                         NewUserEmailAddress=user_email,
                                         NewUserPhoneNumber=user_phone,
                                         OnError=errorcodes.INFACMD_CREATE_USER_FAILED
                                         )
    result = manageSecurity.ManageSecurity.manage(user)

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv[1:])



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# File handling
# @Since: 23-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190323.0 - JBE - Initial

import contextlib, os, shutil
from supporting import generalSettings, randomize


def removefile(filename):
    with contextlib.suppress(FileNotFoundError):
        os.remove(filename)


def copy_file(source, target):
    with contextlib.suppress(FileExistsError):
        shutil.copy2(source, target)


def copy_files(source, target):
    with contextlib.suppress(FileExistsError):
        for file in os.listdir(source):
            shutil.copy2(source + '/' + file, target)


def create_directory(directory):
    os.makedirs(directory, exist_ok=True)  # succeeds even if directory exists.


def generate_tmp_filename():
    generalSettings.getenvvars()
    tmp_file_name = generalSettings.logDir + '/' + randomize.randomStringDigits() + '.tmp'
    return tmp_file_name


def convert_content_to_array(file):
    with open(file, 'r') as f:
        list = f.read().splitlines()
    return list




#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

##
# Informatica App Checks
# @Since: 17-FEB-2020
# @Author: Jac. Beekers
# @Version: 20200217.0 - JBE - Initial

import supporting.errorcodes as err
import supporting, logging
from cicd.informatica import infaSettings as settings
from pathlib import Path

logger = logging.getLogger(__name__)


def infa_app_checks():
    thisproc = "infa_app_checks"
    supporting.log(logger, logging.DEBUG, thisproc, 'started')
    result = err.OK

    if not settings.infa_app_deploylist:
        supporting.log(logger, err.IGNORE.level, thisproc, err.NO_DEPLOYLIST.message)
        supporting.log(logger, err.IGNORE.level, thisproc, "Assuming Informatica Apps are NOT part of the solution.")
        result = err.IGNORE
    else:
        deploylistFile = Path(settings.infa_app_deploylist)
        if not deploylistFile.is_file():
            supporting.log(logger, err.IGNORE.level, thisproc,
                           "deploylist is >" + settings.infa_app_deploylist + "<. "
                           + err.DEPLOYLIST_NF.message + " - Informatica Apps IGNORED.")
            result = err.IGNORE

    supporting.log(logger, logging.DEBUG, thisproc, 'completed with >' + str(result.rc) + "<.")
    return result


def infa_deploy_checks():
    thisproc = "infadeploychecks"
    supporting.log(logger, logging.DEBUG, thisproc, 'started')

    # for now the checks for deploys and artifacts are the same
    result = infa_app_checks()

    supporting.log(logger, logging.DEBUG, thisproc, 'completed with >' + str(result.rc) + "<.")
    return result



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# Supporting modules
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190414.0 - JBE - Initial

import logging, datetime, os
import supporting.generalConstants as generalConstants
import supporting
import sys

logger = logging.getLogger(__name__)

now = datetime.datetime.now()


def configurelogger(mainProc, console=True):

    logdir = os.environ.get(generalConstants.varLogDir, generalConstants.DEFAULT_LOGDIR)
    logging.basicConfig(filename= logdir +"/" + now.strftime("%Y%m%d-%H%M%S.%f") + '-' + mainProc + '.log'
                        , level=logging.DEBUG
                        , format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    # nice for Azure DevOps, but not for Airflow
    if console:
        console = logging.StreamHandler()
        console.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        # tell the handler to use this format
        console.setFormatter(formatter)
        # add the handler to the root logger
        logging.getLogger('').addHandler(console)


    ResultDir = os.environ.get(generalConstants.varResultDir, generalConstants.DEFAULT_RESULTDIR)
    ResultFileName = ResultDir + "/" + now.strftime("%Y%m%d-%H%M%S.%f") + '-' + mainProc + '.result'

    resultlogger = logging.getLogger('result_logger')
    resultlogger.setLevel(logging.INFO)
    # create file handler which logs even debug messages
    fh = logging.FileHandler(ResultFileName)
    fh.setLevel(logging.INFO)
    # create formatter and add it to the handler
    formatter = logging.Formatter('%(message)s')
    fh.setFormatter(formatter)
    # add the handlers to logger
    resultlogger.addHandler(fh)

    return resultlogger


def log(logger, level, area, message):

    logger.log(level, area + " - " + message)
    return

def writeresult(resultlogger, result):
    resultlogger.info('RC=' +str(result.rc) )
    resultlogger.info('CODE=' + result.code )
    resultlogger.info('MSG=' + result.message )
    resultlogger.info('RESOLUTION=' + result.resolution )
    resultlogger.info('AREA=' + result.area )
    resultlogger.info('ERRLEVEL=' + str(result.level) )


def exitscript(resultlogger, result):
    writeresult(resultlogger, result)
    sys.exit(result.rc)


def logentireenv():
    thisproc = "logentireenv"
    for param in os.environ.keys():
        supporting.log(logger, logging.DEBUG, thisproc,"%30s %s" % (param, os.environ[param]))




#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

def add_part(part, separator=' '):
    global masked_string
    masked_string = masked_string + separator + part


def mask_password(input_string, separator=' '):
    next_is_password = False
    global masked_string

    masked_string = ""
    splitted_string = input_string.split(separator)

    for s in splitted_string:
        l = ""
        if next_is_password:
            add_part('***')
            next_is_password = False
        else:
            l = s.lower()
            if l.__contains__("password"):
                next_is_password = True
            add_part(s)

    return masked_string



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#
import supporting.errorcodes as err
from zipfile import ZipFile
import os
import logging
import supporting
import fnmatch

logger = logging.getLogger(__name__)


def generate_zip(basedirectory, directory, zipFileName, filter='*', suppress_extension='7Al!#%ˆˆ'):
    thisproc = "generate_zip"

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Creating new zip >" + zipFileName + "<...")
    # create a ZipFile object
    with ZipFile(zipFileName, 'w') as zipObj:
        additemto_zip(zipObj, basedirectory, directory, filter, suppress_extension)
    supporting.log(logger, logging.DEBUG, thisproc,
                   "Done.")

    return err.OK


def addto_zip(basedirectory, directory, zipFileName, filter='*', suppress_extension='7Al!#%ˆˆ'):
    thisproc = "addto_zip"

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Adding to zip >" + zipFileName + "<...")

    # create a ZipFile object
    with ZipFile(zipFileName, 'a') as zipObj:
        additemto_zip(zipObj, basedirectory, directory, filter, suppress_extension)

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Done.")

    return err.OK


def additemto_zip(zipObj, basedirectory, item, filter='*', suppress_extension='7Al!#%ˆˆ'):
    thisproc = "additemto_zip"

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Adding >" + item + "< ...")

    for folderName, subfolders, filenames in os.walk(item):
        for filename in filenames:
            if filename.endswith('.' + suppress_extension):
                supporting.log(logger, logging.DEBUG, thisproc,
                               "Ignoring >" + filename + "< as it has the extension >" + suppress_extension + "<.")
            else:
                if fnmatch.fnmatch(filename, filter):
                    filePath = os.path.join(folderName, filename)
                    # Add file to zip
                    archive_name = filePath[len(basedirectory):]
                    supporting.log(logger, logging.DEBUG, thisproc,
                                   "Adding >" + filePath + "< to zip as >" + archive_name + "<.")
                    zipObj.write(filePath, archive_name)
                else:
                    supporting.log(logger, logging.DEBUG, thisproc,
                                   ">" + filename + "< was not added to zip as it does not match pattern >" + filter + "<.")

    supporting.log(logger, logging.DEBUG, thisproc,
                   "Done adding >" + item + "< ...")

    return err.OK



"""IDQ Importer Exporter

This script defines Import and Export functions through which it can communicate with
a Informatica Model Repository.

It also provides some related functions, such as:
	- Create IDQ folder
	- Check in IDQ components

    Parts by Laurens Verhoeven
    Parts by Jac. Beekers
    @Version: 20190412.0  - JBE - Initial version to work with deploy lists
    @License: MIT
"""

#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

import datetime
import supporting, logging

from cicd.informatica import buildCommand
from cicd.informatica import executeInfacmd
from cicd.informatica import infaConstants as constants

logger = logging.getLogger(__name__)


def import_infadeveloper(**KeyWordArguments):
    """Import IDQ Components"""

    KeyWordArguments["Tool"] = "Import"
    ImportCommand = buildCommand.build(**KeyWordArguments)

    result = executeInfacmd.execute(ImportCommand, constants.DEPLOYARTIFACT)

    return result


def export_infadeveloper(**KeyWordArguments):
    thisproc = "export_infadeveloper"

    KeyWordArguments["Tool"] = "Export"
    ExportCommand = buildCommand.build(**KeyWordArguments)

    supporting.log(logger, logging.INFO, thisproc, "ExportCommand is >" + ExportCommand + "<.")
    result = executeInfacmd.execute(ExportCommand, constants.CREATEARTIFACT)

    return result


def CreateFolder(**KeyWordArguments):
    """Create IDQ Folder"""

    KeyWordArguments["Tool"] = "CreateFolder"

    CreateFolder = buildCommand.build(**KeyWordArguments)

    output, error = executeInfacmd.execute(CreateFolder)

    return (output, error)


def ListCheckedOutObjects(**KeyWordArguments):
    thisproc = "ListCheckedOutObjects"
    """ List Components that are currently checked out """

    KeyWordArguments["Tool"] = "ListCheckOutObjects"
    ListCheckedOutCommand = buildCommand.build(**KeyWordArguments)
    output, error = executeInfacmd.execute(ListCheckedOutCommand)

    # The output is in the form of one object per line, with properties spearated by a comma + space.
    # To filter out irrelevant lines, such as "Command succesful", we keep only line that start with "MRS_PATH="
    OutputLines = output.splitlines()
    OutputKeyValuePairLines = [Properties.split(", ") for Properties in OutputLines if
                               Properties.startswith("MRS_PATH=")]

    # ObjectsOLD = [[KVPair.split("=", 1) for KVPair in Line] for Line in OutputKeyValuePairLines]

    # Each object is a dictionary, with properties as keys
    # Since the date field has a comma in it, its not parsed properly. For this reason we need the len == 2 filter
    # If the date is required, the parsing of the output should be adjusted
    Objects = [dict(KVPair.split("=") for KVPair in Line if len(KVPair.split("=")) == 2) for Line in
               OutputKeyValuePairLines]

    supporting.log(logger, logging.DEBUG, thisproc, output)

    return Objects


def CheckIn(**KeyWordArguments):
    """Check-in IDQ Components"""

    KeyWordArguments["Tool"] = "CheckIn"
    CheckInCommand = buildCommand.build(**KeyWordArguments)
    output, error = executeInfacmd.execute(CheckInCommand)

    return (output, error)


def CheckInMutiple(**KeyWordArguments):
    thisproc = "CheckInMultiple"
    """ Check in Multiple IDQ components """
    for key, value in KeyWordArguments.items():
        if key == "MultipleObjectPaths":
            ObjectPaths = KeyWordArguments["MultipleObjectPaths"]

    KeyWordArguments["Tool"] = "CheckIn"

    CheckInCommands = []
    for ObjectPathName in ObjectPaths:
        KeyWordArguments["ObjectPathName"] = ObjectPathName
        CheckInCommands.append(buildCommand.build(**KeyWordArguments))

    CheckInAllCommand = "\n".join(CheckInCommands)

    timebefore = datetime.datetime.now()
    output, error = executeInfacmd.execute(CheckInAllCommand)
    timeafter = datetime.datetime.now()
    duration = timeafter - timebefore

    supporting.log(logging.DEBUG, thisproc,
                   "Infacmd took " + str(duration) + " seconds to check-in " + str(len(ObjectPaths)) + " objects")

    # output, error = (CheckInAllCommand, 0)

    return (output, error)


def create_iar_file(**KeyWordArguments):
    thisproc = "create_iar_file"

    KeyWordArguments["Tool"] = "CreateIAR"
    create_command = buildCommand.build(**KeyWordArguments)

    supporting.log(logger, logging.INFO, thisproc, "Command is >" + create_command + "<.")
    result = executeInfacmd.execute(create_command, constants.CREATEARTIFACT)

    return result


def deploy_iar_file(**KeyWordArguments):
    thisproc = "deploy_iar_file"

    KeyWordArguments["Tool"] = "DeployIAR"
    deploy_command = buildCommand.build(**KeyWordArguments)

    supporting.log(logger, logging.INFO, thisproc, "Command is >" + deploy_command + "<.")
    result = executeInfacmd.execute(deploy_command, constants.DEPLOYARTIFACT)

    return result


def redeploy_iar_file(**KeyWordArguments):
    thisproc = "redeploy_iar_file"

    KeyWordArguments["Tool"] = "RedeployIAR"
    deploy_command = buildCommand.build(**KeyWordArguments)

    supporting.log(logger, logging.INFO, thisproc, "Command is >" + deploy_command + "<.")
    result = executeInfacmd.execute(deploy_command, constants.DEPLOYARTIFACT)

    return result


def stop_app(**KeyWordArguments):
    thisproc = "stop_app"

    KeyWordArguments["Tool"] = "StopApp"
    deploy_command = buildCommand.build(**KeyWordArguments)

    supporting.log(logger, logging.INFO, thisproc, "Command is >" + deploy_command + "<.")
    result = executeInfacmd.execute(deploy_command, constants.DEPLOYARTIFACT)

    return result



import setuptools
import re
import ast

# https://packaging.python.org/tutorials/packaging-projects/

with open("README.md", "r") as fh:
    long_description = fh.read()

version_file = 'version/__init__.py'
tmp_version_file = 'temp/_tmp_version.tmp'

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open(version_file, 'rb') as f:
    for line in f:
        line=line.strip()
        if line:
            line=line.decode('utf-8')
            reg_comment = re.compile(r'^#.*')
            m = reg_comment.search(line)
            if not m:
               result_search = _version_re.search(line)
               version = result_search.group(1)
               version = version.strip('\"')
#               print("version is >" + version + "<.")
               main_version, sub_version, fix_version = version.split(".")
               fix_number = int(fix_version) + 1
               new_version = main_version +"." + sub_version + "." + str(fix_number)
#               print("version will be >" + new_version + "<.")
               with open(tmp_version_file, 'wb') as t:
                    out_line ='__version__ = "' + new_version + '"\n'
                    t.write(out_line.encode('utf-8'))
 

setuptools.setup(
    name='devops-informatica-tools',
    version=new_version,
    author='Jac. Beekers',
    author_email='jactools@consag.nl',
    description='DevOps and CI-CD Pipeline scripts for Informatica Platform related projects',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/consag/devops-informatica-tools',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.6'
)




#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# Constants for Database artifacts and deploys
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20190410.0 - JBE - Initial
# @Version: 20190817.0 - JBE - Added user/password functionality

# Database artifacts
varOracleSchemaName='ORACLE_SCHEMA'
varOracleDeployList='ORACLE_DEPLOYLIST'
varSourceSqlDir='SOURCE_SQLDIR'
varTargetSqlDir='TARGET_SQLDIR'
varSqlPrefix = 'SQL_PREFIX'
varOracleDatabaseUser = 'ORACLE_USER'
varDatabaseUserPassword = 'ORACLE_PASSWORD' # encrypted
varOracleTNSName = 'ORACLE_TNS'

##
# Database artifact defaults
DEFAULT_SOURCE_SQLDIR ='.'
DEFAULT_TARGET_SQLDIR ='.'
DEFAULT_ORACLE_DEPLOYLIST ='oracle_deploylist.txt'
DEFAULT_SQL_DEPLOYLIST ='sqlserver_deploylist.txt'
DEFAULT_DB2_DEPLOYLIST ='db2_deploylist.txt'
DEFAULT_SQL_PREFIX = ''

NOT_SET ='NOT_SET'



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

import logging, datetime, supporting
from supporting import errorcodes
from cicd.informatica import infaSettings
from cicd.informatica import infaConstants
from supporting import generalSettings
from cicd.informatica import manageConnection
import sys

now = datetime.datetime.now()
result = errorcodes.OK


def main(argv):
    thisproc = "MAIN"
    mainProc = 'importConnections'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    generalSettings.getenvvars()

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

    if len(argv) < 2:
        supporting.log(logger, logging.ERROR, thisproc, 'No import file and/or control file provided.')
        result = errorcodes.INFACMD_NOIMPORTFILENAME
        supporting.exitscript(resultlogger, result)

    #   mandatory
    input_file = argv[0] if len(argv) > 0 else infaConstants.DEFAULT_IMPORT_CONNECTIONSFILE
    import_control_file = argv[1] if len(argv) > 1 else ""
    # optional

    infaSettings.getinfaenvvars()
    #    infaSettings.outinfaenvvars()

    connection = manageConnection.ManageConnection(Tool="ImportConnections",
                                                   Domain=infaSettings.sourceDomain,
                                                   ImportControlfile=import_control_file,
                                                   ImportFilePath=input_file,
                                                   OnError=errorcodes.INFACMD_EXPORT_CONN_FAILED
                                                   )

    result = connection.manage()

    supporting.log(logger, logging.DEBUG, thisproc, 'Completed with return code >' + str(result.rc)
                   + '< and result code >' + result.code + "<.")
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv[1:])



#  MIT License
# 
#  Copyright (c) 2019 Jac. Beekers
# 
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
# 

##
# schedulerSettings
# @Since: 25-OCT-2019
# @Author: Jac. Beekers
# @Version: 20191025.0 - JBE - Initial
##

from cicd.scheduler import schedulerConstants as constants
import supporting, os, logging
import supporting.generalSettings as generalsettings
from supporting.generalSettings import completePath

logger = logging.getLogger(__name__)

sourceschedulerdir = constants.DEFAULT_SOURCE_SCHEDULERDIR
sourceschedulertypedir = constants.DEFAULT_SOURCE_SCHEDULER_TYPEDIR
targetschedulerdir = constants.DEFAULT_TARGET_SCHEDULERDIR
targetschedulertypedir = constants.DEFAULT_TARGET_SCHEDULER_TYPEDIR
schedulerdeploylist = constants.DEFAULT_SCHEDULER_DEPLOYLIST

def getschedulerenvvars():
    thisproc = "getschedulerenvvars"
    global schedulerdeploylist, sourceschedulerdir, targetschedulerdir, sourceschedulertypedir, targetschedulertypedir
    supporting.log(logger, logging.DEBUG, thisproc, 'started')

    schedulerdeploylist = completePath(
        os.environ.get(constants.varSchedulerDeployList, constants.DEFAULT_SCHEDULER_DEPLOYLIST),
        generalsettings.sourceDir)
    sourceschedulerdir = completePath(
        os.environ.get(constants.varSourceSchedulerDir, constants.DEFAULT_SOURCE_SCHEDULERDIR),
        generalsettings.sourceDir)
    sourceschedulertypedir = completePath(
        os.environ.get(constants.varSourceSchedulerTypeDir, constants.DEFAULT_SOURCE_SCHEDULER_TYPEDIR),
        generalsettings.sourceDir)
    targetschedulerdir = completePath(
        os.environ.get(constants.varTargetSchedulerDir, constants.DEFAULT_TARGET_SCHEDULERDIR),
        generalsettings.sourceDir)
    targetschedulertypedir = completePath(
        os.environ.get(constants.varTargetSchedulerTypeDir, constants.DEFAULT_TARGET_SCHEDULER_TYPEDIR),
        generalsettings.sourceDir)


def outschedulerenvvars():
    thisproc = "outschedulerenvvars"
    supporting.log(logger, logging.INFO, thisproc, 'schedulerdeploylist is >' + schedulerdeploylist + "<.")
    supporting.log(logger, logging.INFO, thisproc, 'sourceschedulerdir is >' + sourceschedulerdir + "<.")
    supporting.log(logger, logging.INFO, thisproc, 'sourceschedulertypedir is >' + sourceschedulertypedir + "<.")
    supporting.log(logger, logging.INFO, thisproc, 'targetschedulerdir is >' + targetschedulerdir + "<.")
    supporting.log(logger, logging.INFO, thisproc, 'targetschedulertypedir is >' + targetschedulertypedir + "<.")



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

##
# generalSettings
# @Since: 22-MAR-2019
# @Author: Jac. Beekers
# @Version: 20200301.0 - JBE - Added DO_NOT_RUN

import os, logging
import supporting
import supporting.generalConstants as constants

logger = logging.getLogger(__name__)

"""defaults"""
logDir = constants.DEFAULT_LOGDIR
resultDir = constants.DEFAULT_RESULTDIR
artifactDir = constants.DEFAULT_ARTIFACTDIR
configDir = constants.DEFAULT_CONFIGDIR
sourceDir = constants.DEFAULT_SOURCEDIR
do_not_run = constants.DEFAULT_DO_NOT_RUN

def getenvvars():
    thisproc = "getenvvars"
    global logDir, resultDir, artifactDir, configDir, sourceDir, releaseID, do_not_run

    supporting.log(logger, logging.DEBUG, thisproc, 'started')

    logDir = os.environ.get(constants.varLogDir, constants.DEFAULT_LOGDIR)
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir set to >' + logDir + "<.")

    resultDir = os.environ.get(constants.varResultDir, constants.DEFAULT_RESULTDIR)
    supporting.log(logger, logging.DEBUG, thisproc, 'resultDir set to >' + resultDir + "<.")

    artifactDir = os.environ.get(constants.varArtifactDir, constants.DEFAULT_ARTIFACTDIR)
    supporting.log(logger, logging.DEBUG, thisproc, 'artifactDir set to >' + artifactDir + "<.")

    configDir = os.environ.get(constants.varConfigDir, constants.DEFAULT_CONFIGDIR)
    supporting.log(logger, logging.DEBUG, thisproc, 'configDir set to >' + configDir + "<.")

    sourceDir = os.environ.get(constants.varSourceDir, constants.DEFAULT_SOURCEDIR)
    supporting.log(logger, logging.DEBUG, thisproc, 'sourceDir set to >' + sourceDir + "<.")

    releaseID = os.environ.get(constants.varReleaseId, constants.DEFAULT_RELEASEID)
    supporting.log(logger, logging.DEBUG, thisproc, 'releaseID set to >' + releaseID + "<.")

    do_not_run = os.environ.get(constants.varDoNotRun, constants.DEFAULT_DO_NOT_RUN)
    supporting.log(logger, logging.DEBUG, thisproc, 'do_not_run set to >' + do_not_run + "<.")

    supporting.log(logger, logging.DEBUG, thisproc, 'completed')


getenvvars()


def completePath(foundPath, prefixPath):
    if foundPath.startswith("/"):
        return foundPath
    else:
        return prefixPath +"/" + foundPath


#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

from supporting import log
import logging
import supporting.errorcodes as errorcodes
import subprocess
import os

from supporting.mask_password import mask_password

logger = logging.getLogger(__name__)
entrynr =0

errorMessage =errorcodes.INFACMD_FAILED.message

def execute(command):
    env = {**os.environ, 'NO_ENV': 'TRUE'}
    execute(command, env)


def execute(command, env, pre_command = None):
    """Execute the command on the underlying system, capturing the output generated by the command."""
    thisproc = "execute"
    process = ""
    result = errorcodes.OK

    if pre_command is None:
        log(logger, logging.INFO, thisproc, "No pre_command provided.")
    else:
        log(logger, logging.INFO, thisproc, "pre_command is >" + pre_command + "<.")
        command = pre_command + ' && '+ command

    log(logger, logging.INFO, thisproc, "Executing command >" + mask_password(command) + "<.")

    pipes = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, env=env)
    pipes.wait()
    std_out, std_err = pipes.communicate()

    if pipes.returncode == 0:
        log(logger, logging.INFO, thisproc, std_out.decode("utf-8"))
    else:
        result = errorcodes.COMMAND_FAILED
        result.message = std_out.decode("utf-8") + std_err.decode("utf-8")
        log(logger, logging.ERROR, thisproc, result.message)

    return result



#  MIT License
#
#  Copyright (c) 2019 Jac. Beekers
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#

#  MIT License
#
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#
#

from supporting import log
import supporting
import logging
from cicd.database.utilities import OracleUtils
# from os import listdir
import sys, argparse
from supporting import generalSettings
import cicd.database.dbSettings as dbSettings
import glob
import supporting.errorcodes as err

logger = logging.getLogger(__name__)


class DeployOracle:
    """Deploys a previously built Oracle package"""

    def __init__(self, schema):
        thisproc = "init"
        self.logger = logging.getLogger(__name__)
        self.database_schema = schema
        dbSettings.getdbenvvars()
        self.database_tns_name, self.database_schema, self.database_user, self.database_user_password = dbSettings.getschemaenvvars(
            schema)
        self.sqldir = dbSettings.targetsqldir
        dbSettings.outdbenvvars()
        dbSettings.outschemaenvvars()

    #        log(self.logger, logging.DEBUG, thisproc, 'database_tns_name is >' + self.database_tns_name +"<.")
    #        log(self.logger, logging.DEBUG, thisproc, 'database_schema is >' + self.database_schema +"<.")
    #        log(self.logger, logging.DEBUG, thisproc, 'database_user is >' + self.database_user + "<.")
    #        if self.database_user_password is None or self.database_user_password == dbConstants.NOT_SET:
    #            log(self.logger, logging.WARN, thisproc, 'database_user_password is empty.')
    #        else:
    #            log(self.logger, logging.DEBUG, thisproc, 'database_user_password has a value.')

    def deployArtifact(self):
        thisproc = "deployArtifact"
        overall_result = err.OK
        log(self.logger, logging.INFO, thisproc, "deployArtifact started.")
        log(self.logger, logging.DEBUG, thisproc, "sqldir is >" + self.sqldir + "<")
        log(self.logger, logging.DEBUG, thisproc, "database_schema is >" + self.database_schema + "<.")
        schema_directory = self.sqldir + '/' + self.database_schema
        log(self.logger, logging.DEBUG, thisproc, "schema_directory is >" + schema_directory + "<.")
        sql_files = [f for f in glob.glob(schema_directory + "**/*.sql")]
        log(self.logger, logging.DEBUG, thisproc, "sql_files found >" + str(len(sql_files)) + "<.")
        sql_files.sort()
        for sqlfile in sql_files:
            if sqlfile[-4:] != ".sql":
                log(self.logger, logging.INFO, thisproc, "Ignored non-sql file >" + sqlfile + "<.")
                continue
            log(self.logger, logging.INFO, thisproc, "Processing sql file >" + sqlfile + "<.")
            oracle_util = OracleUtils.OracleUtilities(self.database_user
                                               , self.database_user_password
                                               , self.database_tns_name
                                               , 'REPORT'
                                               , self.database_schema + '_sqloutput.log')
            sqlplus_result = oracle_util.run_sqlplus(sqlfile)
            if sqlplus_result.rc != 0:
                log(self.logger, logging.WARNING, thisproc, "sqlplus returned >" + sqlplus_result.code + "<.")
                overall_result = sqlplus_result

        log(self.logger, logging.INFO, thisproc, "deployArtifact completed with >" + overall_result.code + "<.")
        return overall_result

def parse_the_arguments(argv):
    """Parses the provided arguments and exits on an error.
    Use the option -h on the command line to get an overview of the required and optional arguments.
     """
    parser = argparse.ArgumentParser()
    parser.add_argument("-s", "--schema", required=True, action="store", dest="schema_name",
                        help="Deploy mentioned schema.")
    args = parser.parse_args(argv)

    return args


def main(argv):
    """Deploy the Oracle artifact to the target environment.
    Usage: deployOracle.py [-h] -s SCHEMA_NAME
    The module uses environment variables to steer the deployment, like target Oracle database, connections and such.
    For more information check the deployOracle docs.
    """
    thisproc = "MAIN"
    mainProc = 'deployOracle'

    resultlogger = supporting.configurelogger(mainProc)
    logger = logging.getLogger(mainProc)

    args = parse_the_arguments(argv)

    generalSettings.getenvvars()

    supporting.log(logger, logging.DEBUG, thisproc, 'Started')
    supporting.log(logger, logging.DEBUG, thisproc, 'logDir is >' + generalSettings.logDir + "<.")

    schema = args.schema_name
    supporting.log(logger, logging.INFO, thisproc, "Schema to be deployed is >" + schema + "<.")
    depl = DeployOracle(schema)
    result = depl.deployArtifact()
    supporting.exitscript(resultlogger, result)


if __name__ == '__main__':
    main(sys.argv[1:])
